# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.rpc import code_pb2  # type: ignore
from google.type import date_pb2  # type: ignore
from google.type import timeofday_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.storagetransfer.v1",
    manifest={
        "GoogleServiceAccount",
        "AwsAccessKey",
        "AzureCredentials",
        "ObjectConditions",
        "GcsData",
        "AwsS3Data",
        "AzureBlobStorageData",
        "HttpData",
        "PosixFilesystem",
        "AwsS3CompatibleData",
        "S3CompatibleMetadata",
        "AgentPool",
        "TransferOptions",
        "TransferSpec",
        "MetadataOptions",
        "TransferManifest",
        "Schedule",
        "EventStream",
        "TransferJob",
        "ErrorLogEntry",
        "ErrorSummary",
        "TransferCounters",
        "NotificationConfig",
        "LoggingConfig",
        "TransferOperation",
    },
)


class GoogleServiceAccount(proto.Message):
    r"""Google service account

    Attributes:
        account_email (str):
            Email address of the service account.
        subject_id (str):
            Unique identifier for the service account.
    """

    account_email: str = proto.Field(
        proto.STRING,
        number=1,
    )
    subject_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class AwsAccessKey(proto.Message):
    r"""AWS access key (see `AWS Security
    Credentials <https://docs.aws.amazon.com/general/latest/gr/aws-security-credentials.html>`__).

    For information on our data retention policy for user credentials,
    see `User
    credentials </storage-transfer/docs/data-retention#user-credentials>`__.

    Attributes:
        access_key_id (str):
            Required. AWS access key ID.
        secret_access_key (str):
            Required. AWS secret access key. This field
            is not returned in RPC responses.
    """

    access_key_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    secret_access_key: str = proto.Field(
        proto.STRING,
        number=2,
    )


class AzureCredentials(proto.Message):
    r"""Azure credentials

    For information on our data retention policy for user credentials,
    see `User
    credentials </storage-transfer/docs/data-retention#user-credentials>`__.

    Attributes:
        sas_token (str):
            Required. Azure shared access signature (SAS).

            For more information about SAS, see `Grant limited access to
            Azure Storage resources using shared access signatures
            (SAS) <https://docs.microsoft.com/en-us/azure/storage/common/storage-sas-overview>`__.
    """

    sas_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ObjectConditions(proto.Message):
    r"""Conditions that determine which objects are transferred. Applies
    only to Cloud Data Sources such as S3, Azure, and Cloud Storage.

    The "last modification time" refers to the time of the last change
    to the object's content or metadata — specifically, this is the
    ``updated`` property of Cloud Storage objects, the ``LastModified``
    field of S3 objects, and the ``Last-Modified`` header of Azure
    blobs.

    Transfers with a
    [PosixFilesystem][google.storagetransfer.v1.PosixFilesystem] source
    or destination don't support ``ObjectConditions``.

    Attributes:
        min_time_elapsed_since_last_modification (google.protobuf.duration_pb2.Duration):
            Ensures that objects are not transferred until a specific
            minimum time has elapsed after the "last modification time".
            When a
            [TransferOperation][google.storagetransfer.v1.TransferOperation]
            begins, objects with a "last modification time" are
            transferred only if the elapsed time between the
            [start_time][google.storagetransfer.v1.TransferOperation.start_time]
            of the ``TransferOperation`` and the "last modification
            time" of the object is equal to or greater than the value of
            min_time_elapsed_since_last_modification`. Objects that do
            not have a "last modification time" are also transferred.
        max_time_elapsed_since_last_modification (google.protobuf.duration_pb2.Duration):
            Ensures that objects are not transferred if a specific
            maximum time has elapsed since the "last modification time".
            When a
            [TransferOperation][google.storagetransfer.v1.TransferOperation]
            begins, objects with a "last modification time" are
            transferred only if the elapsed time between the
            [start_time][google.storagetransfer.v1.TransferOperation.start_time]
            of the ``TransferOperation``\ and the "last modification
            time" of the object is less than the value of
            max_time_elapsed_since_last_modification`. Objects that do
            not have a "last modification time" are also transferred.
        include_prefixes (MutableSequence[str]):
            If you specify ``include_prefixes``, Storage Transfer
            Service uses the items in the ``include_prefixes`` array to
            determine which objects to include in a transfer. Objects
            must start with one of the matching ``include_prefixes`` for
            inclusion in the transfer. If
            [exclude_prefixes][google.storagetransfer.v1.ObjectConditions.exclude_prefixes]
            is specified, objects must not start with any of the
            ``exclude_prefixes`` specified for inclusion in the
            transfer.

            The following are requirements of ``include_prefixes``:

            -  Each include-prefix can contain any sequence of Unicode
               characters, to a max length of 1024 bytes when
               UTF8-encoded, and must not contain Carriage Return or
               Line Feed characters. Wildcard matching and regular
               expression matching are not supported.

            -  Each include-prefix must omit the leading slash. For
               example, to include the object
               ``s3://my-aws-bucket/logs/y=2015/requests.gz``, specify
               the include-prefix as ``logs/y=2015/requests.gz``.

            -  None of the include-prefix values can be empty, if
               specified.

            -  Each include-prefix must include a distinct portion of
               the object namespace. No include-prefix may be a prefix
               of another include-prefix.

            The max size of ``include_prefixes`` is 1000.

            For more information, see `Filtering objects from
            transfers </storage-transfer/docs/filtering-objects-from-transfers>`__.
        exclude_prefixes (MutableSequence[str]):
            If you specify ``exclude_prefixes``, Storage Transfer
            Service uses the items in the ``exclude_prefixes`` array to
            determine which objects to exclude from a transfer. Objects
            must not start with one of the matching ``exclude_prefixes``
            for inclusion in a transfer.

            The following are requirements of ``exclude_prefixes``:

            -  Each exclude-prefix can contain any sequence of Unicode
               characters, to a max length of 1024 bytes when
               UTF8-encoded, and must not contain Carriage Return or
               Line Feed characters. Wildcard matching and regular
               expression matching are not supported.

            -  Each exclude-prefix must omit the leading slash. For
               example, to exclude the object
               ``s3://my-aws-bucket/logs/y=2015/requests.gz``, specify
               the exclude-prefix as ``logs/y=2015/requests.gz``.

            -  None of the exclude-prefix values can be empty, if
               specified.

            -  Each exclude-prefix must exclude a distinct portion of
               the object namespace. No exclude-prefix may be a prefix
               of another exclude-prefix.

            -  If
               [include_prefixes][google.storagetransfer.v1.ObjectConditions.include_prefixes]
               is specified, then each exclude-prefix must start with
               the value of a path explicitly included by
               ``include_prefixes``.

            The max size of ``exclude_prefixes`` is 1000.

            For more information, see `Filtering objects from
            transfers </storage-transfer/docs/filtering-objects-from-transfers>`__.
        last_modified_since (google.protobuf.timestamp_pb2.Timestamp):
            If specified, only objects with a "last modification time"
            on or after this timestamp and objects that don't have a
            "last modification time" are transferred.

            The ``last_modified_since`` and ``last_modified_before``
            fields can be used together for chunked data processing. For
            example, consider a script that processes each day's worth
            of data at a time. For that you'd set each of the fields as
            follows:

            -  ``last_modified_since`` to the start of the day

            -  ``last_modified_before`` to the end of the day
        last_modified_before (google.protobuf.timestamp_pb2.Timestamp):
            If specified, only objects with a "last
            modification time" before this timestamp and
            objects that don't have a "last modification
            time" are transferred.
    """

    min_time_elapsed_since_last_modification: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=1,
        message=duration_pb2.Duration,
    )
    max_time_elapsed_since_last_modification: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=2,
        message=duration_pb2.Duration,
    )
    include_prefixes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    exclude_prefixes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )
    last_modified_since: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    last_modified_before: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )


class GcsData(proto.Message):
    r"""In a GcsData resource, an object's name is the Cloud Storage
    object's name and its "last modification time" refers to the
    object's ``updated`` property of Cloud Storage objects, which
    changes when the content or the metadata of the object is updated.

    Attributes:
        bucket_name (str):
            Required. Cloud Storage bucket name. Must meet `Bucket Name
            Requirements </storage/docs/naming#requirements>`__.
        path (str):
            Root path to transfer objects.

            Must be an empty string or full path name that ends with a
            '/'. This field is treated as an object prefix. As such, it
            should generally not begin with a '/'.

            The root path value must meet `Object Name
            Requirements </storage/docs/naming#objectnames>`__.
    """

    bucket_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    path: str = proto.Field(
        proto.STRING,
        number=3,
    )


class AwsS3Data(proto.Message):
    r"""An AwsS3Data resource can be a data source, but not a data
    sink. In an AwsS3Data resource, an object's name is the S3
    object's key name.

    Attributes:
        bucket_name (str):
            Required. S3 Bucket name (see `Creating a
            bucket <https://docs.aws.amazon.com/AmazonS3/latest/dev/create-bucket-get-location-example.html>`__).
        aws_access_key (google.cloud.storage_transfer_v1.types.AwsAccessKey):
            Input only. AWS access key used to sign the API requests to
            the AWS S3 bucket. Permissions on the bucket must be granted
            to the access ID of the AWS access key.

            For information on our data retention policy for user
            credentials, see `User
            credentials </storage-transfer/docs/data-retention#user-credentials>`__.
        path (str):
            Root path to transfer objects.

            Must be an empty string or full path name that
            ends with a '/'. This field is treated as an
            object prefix. As such, it should generally not
            begin with a '/'.
        role_arn (str):
            The Amazon Resource Name (ARN) of the role to support
            temporary credentials via ``AssumeRoleWithWebIdentity``. For
            more information about ARNs, see `IAM
            ARNs <https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html#identifiers-arns>`__.

            When a role ARN is provided, Transfer Service fetches
            temporary credentials for the session using a
            ``AssumeRoleWithWebIdentity`` call for the provided role
            using the
            [GoogleServiceAccount][google.storagetransfer.v1.GoogleServiceAccount]
            for this project.
        credentials_secret (str):
            Optional. The Resource name of a secret in Secret Manager.

            The Azure SAS token must be stored in Secret Manager in JSON
            format:

            .. raw:: html

                <pre>{
                 "sas_token" : "<var>SAS_TOKEN</var>"
                }</pre>

            [GoogleServiceAccount][google.storagetransfer.v1.GoogleServiceAccount]
            must be granted ``roles/secretmanager.secretAccessor`` for
            the resource.

            See [Configure access to a source: Microsoft Azure Blob
            Storage]
            (https://cloud.google.com/storage-transfer/docs/source-microsoft-azure#secret_manager)
            for more information.

            If ``credentials_secret`` is specified, do not specify
            [azure_credentials][].

            This feature is in
            `preview <https://cloud.google.com/terms/service-terms#1>`__.

            Format: ``projects/{project_number}/secrets/{secret_name}``
    """

    bucket_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    aws_access_key: "AwsAccessKey" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="AwsAccessKey",
    )
    path: str = proto.Field(
        proto.STRING,
        number=3,
    )
    role_arn: str = proto.Field(
        proto.STRING,
        number=4,
    )
    credentials_secret: str = proto.Field(
        proto.STRING,
        number=7,
    )


class AzureBlobStorageData(proto.Message):
    r"""An AzureBlobStorageData resource can be a data source, but not a
    data sink. An AzureBlobStorageData resource represents one Azure
    container. The storage account determines the `Azure
    endpoint <https://docs.microsoft.com/en-us/azure/storage/common/storage-create-storage-account#storage-account-endpoints>`__.
    In an AzureBlobStorageData resource, a blobs's name is the `Azure
    Blob Storage blob's key
    name <https://docs.microsoft.com/en-us/rest/api/storageservices/naming-and-referencing-containers--blobs--and-metadata#blob-names>`__.

    Attributes:
        storage_account (str):
            Required. The name of the Azure Storage
            account.
        azure_credentials (google.cloud.storage_transfer_v1.types.AzureCredentials):
            Required. Input only. Credentials used to authenticate API
            requests to Azure.

            For information on our data retention policy for user
            credentials, see `User
            credentials </storage-transfer/docs/data-retention#user-credentials>`__.
        container (str):
            Required. The container to transfer from the
            Azure Storage account.
        path (str):
            Root path to transfer objects.

            Must be an empty string or full path name that
            ends with a '/'. This field is treated as an
            object prefix. As such, it should generally not
            begin with a '/'.
        credentials_secret (str):
            Optional. The Resource name of a secret in Secret Manager.

            The Azure SAS token must be stored in Secret Manager in JSON
            format:

            .. raw:: html

                <pre>{
                 "sas_token" : "<var>SAS_TOKEN</var>"
                }</pre>

            [GoogleServiceAccount][google.storagetransfer.v1.GoogleServiceAccount]
            must be granted ``roles/secretmanager.secretAccessor`` for
            the resource.

            See [Configure access to a source: Microsoft Azure Blob
            Storage]
            (https://cloud.google.com/storage-transfer/docs/source-microsoft-azure#secret_manager)
            for more information.

            If ``credentials_secret`` is specified, do not specify
            [azure_credentials][google.storagetransfer.v1.AzureBlobStorageData.azure_credentials].

            This feature is in
            `preview <https://cloud.google.com/terms/service-terms#1>`__.

            Format: ``projects/{project_number}/secrets/{secret_name}``
    """

    storage_account: str = proto.Field(
        proto.STRING,
        number=1,
    )
    azure_credentials: "AzureCredentials" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="AzureCredentials",
    )
    container: str = proto.Field(
        proto.STRING,
        number=4,
    )
    path: str = proto.Field(
        proto.STRING,
        number=5,
    )
    credentials_secret: str = proto.Field(
        proto.STRING,
        number=7,
    )


class HttpData(proto.Message):
    r"""An HttpData resource specifies a list of objects on the web to be
    transferred over HTTP. The information of the objects to be
    transferred is contained in a file referenced by a URL. The first
    line in the file must be ``"TsvHttpData-1.0"``, which specifies the
    format of the file. Subsequent lines specify the information of the
    list of objects, one object per list entry. Each entry has the
    following tab-delimited fields:

    -  **HTTP URL** — The location of the object.

    -  **Length** — The size of the object in bytes.

    -  **MD5** — The base64-encoded MD5 hash of the object.

    For an example of a valid TSV file, see `Transferring data from
    URLs <https://cloud.google.com/storage-transfer/docs/create-url-list>`__.

    When transferring data based on a URL list, keep the following in
    mind:

    -  When an object located at ``http(s)://hostname:port/<URL-path>``
       is transferred to a data sink, the name of the object at the data
       sink is ``<hostname>/<URL-path>``.

    -  If the specified size of an object does not match the actual size
       of the object fetched, the object is not transferred.

    -  If the specified MD5 does not match the MD5 computed from the
       transferred bytes, the object transfer fails.

    -  Ensure that each URL you specify is publicly accessible. For
       example, in Cloud Storage you can [share an object publicly]
       (/storage/docs/cloud-console#_sharingdata) and get a link to it.

    -  Storage Transfer Service obeys ``robots.txt`` rules and requires
       the source HTTP server to support ``Range`` requests and to
       return a ``Content-Length`` header in each response.

    -  [ObjectConditions][google.storagetransfer.v1.ObjectConditions]
       have no effect when filtering objects to transfer.

    Attributes:
        list_url (str):
            Required. The URL that points to the file
            that stores the object list entries. This file
            must allow public access.  Currently, only URLs
            with HTTP and HTTPS schemes are supported.
    """

    list_url: str = proto.Field(
        proto.STRING,
        number=1,
    )


class PosixFilesystem(proto.Message):
    r"""A POSIX filesystem resource.

    Attributes:
        root_directory (str):
            Root directory path to the filesystem.
    """

    root_directory: str = proto.Field(
        proto.STRING,
        number=1,
    )


class AwsS3CompatibleData(proto.Message):
    r"""An AwsS3CompatibleData resource.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        bucket_name (str):
            Required. Specifies the name of the bucket.
        path (str):
            Specifies the root path to transfer objects.

            Must be an empty string or full path name that
            ends with a '/'. This field is treated as an
            object prefix. As such, it should generally not
            begin with a '/'.
        endpoint (str):
            Required. Specifies the endpoint of the
            storage service.
        region (str):
            Specifies the region to sign requests with.
            This can be left blank if requests should be
            signed with an empty region.
        s3_metadata (google.cloud.storage_transfer_v1.types.S3CompatibleMetadata):
            A S3 compatible metadata.

            This field is a member of `oneof`_ ``data_provider``.
    """

    bucket_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    path: str = proto.Field(
        proto.STRING,
        number=2,
    )
    endpoint: str = proto.Field(
        proto.STRING,
        number=3,
    )
    region: str = proto.Field(
        proto.STRING,
        number=5,
    )
    s3_metadata: "S3CompatibleMetadata" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="data_provider",
        message="S3CompatibleMetadata",
    )


class S3CompatibleMetadata(proto.Message):
    r"""S3CompatibleMetadata contains the metadata fields that apply
    to the basic types of S3-compatible data providers.

    Attributes:
        auth_method (google.cloud.storage_transfer_v1.types.S3CompatibleMetadata.AuthMethod):
            Specifies the authentication and
            authorization method used by the storage
            service. When not specified, Transfer Service
            will attempt to determine right auth method to
            use.
        request_model (google.cloud.storage_transfer_v1.types.S3CompatibleMetadata.RequestModel):
            Specifies the API request model used to call the storage
            service. When not specified, the default value of
            RequestModel REQUEST_MODEL_VIRTUAL_HOSTED_STYLE is used.
        protocol (google.cloud.storage_transfer_v1.types.S3CompatibleMetadata.NetworkProtocol):
            Specifies the network protocol of the agent. When not
            specified, the default value of NetworkProtocol
            NETWORK_PROTOCOL_HTTPS is used.
        list_api (google.cloud.storage_transfer_v1.types.S3CompatibleMetadata.ListApi):
            The Listing API to use for discovering
            objects. When not specified, Transfer Service
            will attempt to determine the right API to use.
    """

    class AuthMethod(proto.Enum):
        r"""The authentication and authorization method used by the
        storage service.

        Values:
            AUTH_METHOD_UNSPECIFIED (0):
                AuthMethod is not specified.
            AUTH_METHOD_AWS_SIGNATURE_V4 (1):
                Auth requests with AWS SigV4.
            AUTH_METHOD_AWS_SIGNATURE_V2 (2):
                Auth requests with AWS SigV2.
        """
        AUTH_METHOD_UNSPECIFIED = 0
        AUTH_METHOD_AWS_SIGNATURE_V4 = 1
        AUTH_METHOD_AWS_SIGNATURE_V2 = 2

    class RequestModel(proto.Enum):
        r"""The request model of the API.

        Values:
            REQUEST_MODEL_UNSPECIFIED (0):
                RequestModel is not specified.
            REQUEST_MODEL_VIRTUAL_HOSTED_STYLE (1):
                Perform requests using Virtual Hosted Style.
                Example:
                https://bucket-name.s3.region.amazonaws.com/key-name
            REQUEST_MODEL_PATH_STYLE (2):
                Perform requests using Path Style.
                Example:
                https://s3.region.amazonaws.com/bucket-name/key-name
        """
        REQUEST_MODEL_UNSPECIFIED = 0
        REQUEST_MODEL_VIRTUAL_HOSTED_STYLE = 1
        REQUEST_MODEL_PATH_STYLE = 2

    class NetworkProtocol(proto.Enum):
        r"""The agent network protocol to access the storage service.

        Values:
            NETWORK_PROTOCOL_UNSPECIFIED (0):
                NetworkProtocol is not specified.
            NETWORK_PROTOCOL_HTTPS (1):
                Perform requests using HTTPS.
            NETWORK_PROTOCOL_HTTP (2):
                Not recommended: This sends data in
                clear-text. This is only appropriate within a
                closed network or for publicly available data.
                Perform requests using HTTP.
        """
        NETWORK_PROTOCOL_UNSPECIFIED = 0
        NETWORK_PROTOCOL_HTTPS = 1
        NETWORK_PROTOCOL_HTTP = 2

    class ListApi(proto.Enum):
        r"""The Listing API to use for discovering objects.

        Values:
            LIST_API_UNSPECIFIED (0):
                ListApi is not specified.
            LIST_OBJECTS_V2 (1):
                Perform listing using ListObjectsV2 API.
            LIST_OBJECTS (2):
                Legacy ListObjects API.
        """
        LIST_API_UNSPECIFIED = 0
        LIST_OBJECTS_V2 = 1
        LIST_OBJECTS = 2

    auth_method: AuthMethod = proto.Field(
        proto.ENUM,
        number=1,
        enum=AuthMethod,
    )
    request_model: RequestModel = proto.Field(
        proto.ENUM,
        number=2,
        enum=RequestModel,
    )
    protocol: NetworkProtocol = proto.Field(
        proto.ENUM,
        number=3,
        enum=NetworkProtocol,
    )
    list_api: ListApi = proto.Field(
        proto.ENUM,
        number=4,
        enum=ListApi,
    )


class AgentPool(proto.Message):
    r"""Represents an On-Premises Agent pool.

    Attributes:
        name (str):
            Required. Specifies a unique string that identifies the
            agent pool.

            Format: ``projects/{project_id}/agentPools/{agent_pool_id}``
        display_name (str):
            Specifies the client-specified AgentPool
            description.
        state (google.cloud.storage_transfer_v1.types.AgentPool.State):
            Output only. Specifies the state of the
            AgentPool.
        bandwidth_limit (google.cloud.storage_transfer_v1.types.AgentPool.BandwidthLimit):
            Specifies the bandwidth limit details. If
            this field is unspecified, the default value is
            set as 'No Limit'.
    """

    class State(proto.Enum):
        r"""The state of an AgentPool.

        Values:
            STATE_UNSPECIFIED (0):
                Default value. This value is unused.
            CREATING (1):
                This is an initialization state. During this
                stage, the resources such as Pub/Sub topics are
                allocated for the AgentPool.
            CREATED (2):
                Determines that the AgentPool is created for
                use. At this state, Agents can join the
                AgentPool and participate in the transfer jobs
                in that pool.
            DELETING (3):
                Determines that the AgentPool deletion has
                been initiated, and all the resources are
                scheduled to be cleaned up and freed.
        """
        STATE_UNSPECIFIED = 0
        CREATING = 1
        CREATED = 2
        DELETING = 3

    class BandwidthLimit(proto.Message):
        r"""Specifies a bandwidth limit for an agent pool.

        Attributes:
            limit_mbps (int):
                Bandwidth rate in megabytes per second,
                distributed across all the agents in the pool.
        """

        limit_mbps: int = proto.Field(
            proto.INT64,
            number=1,
        )

    name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=4,
        enum=State,
    )
    bandwidth_limit: BandwidthLimit = proto.Field(
        proto.MESSAGE,
        number=5,
        message=BandwidthLimit,
    )


class TransferOptions(proto.Message):
    r"""TransferOptions define the actions to be performed on objects
    in a transfer.

    Attributes:
        overwrite_objects_already_existing_in_sink (bool):
            When to overwrite objects that already exist
            in the sink. The default is that only objects
            that are different from the source are
            ovewritten. If true, all objects in the sink
            whose name matches an object in the source are
            overwritten with the source object.
        delete_objects_unique_in_sink (bool):
            Whether objects that exist only in the sink should be
            deleted.

            **Note:** This option and
            [delete_objects_from_source_after_transfer][google.storagetransfer.v1.TransferOptions.delete_objects_from_source_after_transfer]
            are mutually exclusive.
        delete_objects_from_source_after_transfer (bool):
            Whether objects should be deleted from the source after they
            are transferred to the sink.

            **Note:** This option and
            [delete_objects_unique_in_sink][google.storagetransfer.v1.TransferOptions.delete_objects_unique_in_sink]
            are mutually exclusive.
        overwrite_when (google.cloud.storage_transfer_v1.types.TransferOptions.OverwriteWhen):
            When to overwrite objects that already exist in the sink. If
            not set, overwrite behavior is determined by
            [overwrite_objects_already_existing_in_sink][google.storagetransfer.v1.TransferOptions.overwrite_objects_already_existing_in_sink].
        metadata_options (google.cloud.storage_transfer_v1.types.MetadataOptions):
            Represents the selected metadata options for
            a transfer job.
    """

    class OverwriteWhen(proto.Enum):
        r"""Specifies when to overwrite an object in the sink when an
        object with matching name is found in the source.

        Values:
            OVERWRITE_WHEN_UNSPECIFIED (0):
                Overwrite behavior is unspecified.
            DIFFERENT (1):
                Overwrites destination objects with the
                source objects, only if the objects have the
                same name but different HTTP ETags or checksum
                values.
            NEVER (2):
                Never overwrites a destination object if a
                source object has the same name. In this case,
                the source object is not transferred.
            ALWAYS (3):
                Always overwrite the destination object with
                the source object, even if the HTTP Etags or
                checksum values are the same.
        """
        OVERWRITE_WHEN_UNSPECIFIED = 0
        DIFFERENT = 1
        NEVER = 2
        ALWAYS = 3

    overwrite_objects_already_existing_in_sink: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    delete_objects_unique_in_sink: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    delete_objects_from_source_after_transfer: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    overwrite_when: OverwriteWhen = proto.Field(
        proto.ENUM,
        number=4,
        enum=OverwriteWhen,
    )
    metadata_options: "MetadataOptions" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="MetadataOptions",
    )


class TransferSpec(proto.Message):
    r"""Configuration for running a transfer.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_data_sink (google.cloud.storage_transfer_v1.types.GcsData):
            A Cloud Storage data sink.

            This field is a member of `oneof`_ ``data_sink``.
        posix_data_sink (google.cloud.storage_transfer_v1.types.PosixFilesystem):
            A POSIX Filesystem data sink.

            This field is a member of `oneof`_ ``data_sink``.
        gcs_data_source (google.cloud.storage_transfer_v1.types.GcsData):
            A Cloud Storage data source.

            This field is a member of `oneof`_ ``data_source``.
        aws_s3_data_source (google.cloud.storage_transfer_v1.types.AwsS3Data):
            An AWS S3 data source.

            This field is a member of `oneof`_ ``data_source``.
        http_data_source (google.cloud.storage_transfer_v1.types.HttpData):
            An HTTP URL data source.

            This field is a member of `oneof`_ ``data_source``.
        posix_data_source (google.cloud.storage_transfer_v1.types.PosixFilesystem):
            A POSIX Filesystem data source.

            This field is a member of `oneof`_ ``data_source``.
        azure_blob_storage_data_source (google.cloud.storage_transfer_v1.types.AzureBlobStorageData):
            An Azure Blob Storage data source.

            This field is a member of `oneof`_ ``data_source``.
        aws_s3_compatible_data_source (google.cloud.storage_transfer_v1.types.AwsS3CompatibleData):
            An AWS S3 compatible data source.

            This field is a member of `oneof`_ ``data_source``.
        gcs_intermediate_data_location (google.cloud.storage_transfer_v1.types.GcsData):
            For transfers between file systems, specifies a Cloud
            Storage bucket to be used as an intermediate location
            through which to transfer data.

            See `Transfer data between file
            systems <https://cloud.google.com/storage-transfer/docs/file-to-file>`__
            for more information.

            This field is a member of `oneof`_ ``intermediate_data_location``.
        object_conditions (google.cloud.storage_transfer_v1.types.ObjectConditions):
            Only objects that satisfy these object
            conditions are included in the set of data
            source and data sink objects.  Object conditions
            based on objects' "last modification time" do
            not exclude objects in a data sink.
        transfer_options (google.cloud.storage_transfer_v1.types.TransferOptions):
            If the option
            [delete_objects_unique_in_sink][google.storagetransfer.v1.TransferOptions.delete_objects_unique_in_sink]
            is ``true`` and time-based object conditions such as 'last
            modification time' are specified, the request fails with an
            [INVALID_ARGUMENT][google.rpc.Code.INVALID_ARGUMENT] error.
        transfer_manifest (google.cloud.storage_transfer_v1.types.TransferManifest):
            A manifest file provides a list of objects to
            be transferred from the data source. This field
            points to the location of the manifest file.
            Otherwise, the entire source bucket is used.
            ObjectConditions still apply.
        source_agent_pool_name (str):
            Specifies the agent pool name associated with
            the posix data source. When unspecified, the
            default name is used.
        sink_agent_pool_name (str):
            Specifies the agent pool name associated with
            the posix data sink. When unspecified, the
            default name is used.
    """

    gcs_data_sink: "GcsData" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="data_sink",
        message="GcsData",
    )
    posix_data_sink: "PosixFilesystem" = proto.Field(
        proto.MESSAGE,
        number=13,
        oneof="data_sink",
        message="PosixFilesystem",
    )
    gcs_data_source: "GcsData" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="data_source",
        message="GcsData",
    )
    aws_s3_data_source: "AwsS3Data" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="data_source",
        message="AwsS3Data",
    )
    http_data_source: "HttpData" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="data_source",
        message="HttpData",
    )
    posix_data_source: "PosixFilesystem" = proto.Field(
        proto.MESSAGE,
        number=14,
        oneof="data_source",
        message="PosixFilesystem",
    )
    azure_blob_storage_data_source: "AzureBlobStorageData" = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="data_source",
        message="AzureBlobStorageData",
    )
    aws_s3_compatible_data_source: "AwsS3CompatibleData" = proto.Field(
        proto.MESSAGE,
        number=19,
        oneof="data_source",
        message="AwsS3CompatibleData",
    )
    gcs_intermediate_data_location: "GcsData" = proto.Field(
        proto.MESSAGE,
        number=16,
        oneof="intermediate_data_location",
        message="GcsData",
    )
    object_conditions: "ObjectConditions" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="ObjectConditions",
    )
    transfer_options: "TransferOptions" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="TransferOptions",
    )
    transfer_manifest: "TransferManifest" = proto.Field(
        proto.MESSAGE,
        number=15,
        message="TransferManifest",
    )
    source_agent_pool_name: str = proto.Field(
        proto.STRING,
        number=17,
    )
    sink_agent_pool_name: str = proto.Field(
        proto.STRING,
        number=18,
    )


class MetadataOptions(proto.Message):
    r"""Specifies the metadata options for running a transfer.

    Attributes:
        symlink (google.cloud.storage_transfer_v1.types.MetadataOptions.Symlink):
            Specifies how symlinks should be handled by
            the transfer. By default, symlinks are not
            preserved. Only applicable to transfers
            involving POSIX file systems, and ignored for
            other transfers.
        mode (google.cloud.storage_transfer_v1.types.MetadataOptions.Mode):
            Specifies how each file's mode attribute
            should be handled by the transfer. By default,
            mode is not preserved. Only applicable to
            transfers involving POSIX file systems, and
            ignored for other transfers.
        gid (google.cloud.storage_transfer_v1.types.MetadataOptions.GID):
            Specifies how each file's POSIX group ID
            (GID) attribute should be handled by the
            transfer. By default, GID is not preserved. Only
            applicable to transfers involving POSIX file
            systems, and ignored for other transfers.
        uid (google.cloud.storage_transfer_v1.types.MetadataOptions.UID):
            Specifies how each file's POSIX user ID (UID)
            attribute should be handled by the transfer. By
            default, UID is not preserved. Only applicable
            to transfers involving POSIX file systems, and
            ignored for other transfers.
        acl (google.cloud.storage_transfer_v1.types.MetadataOptions.Acl):
            Specifies how each object's ACLs should be preserved for
            transfers between Google Cloud Storage buckets. If
            unspecified, the default behavior is the same as
            ACL_DESTINATION_BUCKET_DEFAULT.
        storage_class (google.cloud.storage_transfer_v1.types.MetadataOptions.StorageClass):
            Specifies the storage class to set on objects being
            transferred to Google Cloud Storage buckets. If unspecified,
            the default behavior is the same as
            [STORAGE_CLASS_DESTINATION_BUCKET_DEFAULT][google.storagetransfer.v1.MetadataOptions.StorageClass.STORAGE_CLASS_DESTINATION_BUCKET_DEFAULT].
        temporary_hold (google.cloud.storage_transfer_v1.types.MetadataOptions.TemporaryHold):
            Specifies how each object's temporary hold status should be
            preserved for transfers between Google Cloud Storage
            buckets. If unspecified, the default behavior is the same as
            [TEMPORARY_HOLD_PRESERVE][google.storagetransfer.v1.MetadataOptions.TemporaryHold.TEMPORARY_HOLD_PRESERVE].
        kms_key (google.cloud.storage_transfer_v1.types.MetadataOptions.KmsKey):
            Specifies how each object's Cloud KMS customer-managed
            encryption key (CMEK) is preserved for transfers between
            Google Cloud Storage buckets. If unspecified, the default
            behavior is the same as
            [KMS_KEY_DESTINATION_BUCKET_DEFAULT][google.storagetransfer.v1.MetadataOptions.KmsKey.KMS_KEY_DESTINATION_BUCKET_DEFAULT].
        time_created (google.cloud.storage_transfer_v1.types.MetadataOptions.TimeCreated):
            Specifies how each object's ``timeCreated`` metadata is
            preserved for transfers between Google Cloud Storage
            buckets. If unspecified, the default behavior is the same as
            [TIME_CREATED_SKIP][google.storagetransfer.v1.MetadataOptions.TimeCreated.TIME_CREATED_SKIP].
    """

    class Symlink(proto.Enum):
        r"""Whether symlinks should be skipped or preserved during a
        transfer job.

        Values:
            SYMLINK_UNSPECIFIED (0):
                Symlink behavior is unspecified.
            SYMLINK_SKIP (1):
                Do not preserve symlinks during a transfer
                job.
            SYMLINK_PRESERVE (2):
                Preserve symlinks during a transfer job.
        """
        SYMLINK_UNSPECIFIED = 0
        SYMLINK_SKIP = 1
        SYMLINK_PRESERVE = 2

    class Mode(proto.Enum):
        r"""Options for handling file mode attribute.

        Values:
            MODE_UNSPECIFIED (0):
                Mode behavior is unspecified.
            MODE_SKIP (1):
                Do not preserve mode during a transfer job.
            MODE_PRESERVE (2):
                Preserve mode during a transfer job.
        """
        MODE_UNSPECIFIED = 0
        MODE_SKIP = 1
        MODE_PRESERVE = 2

    class GID(proto.Enum):
        r"""Options for handling file GID attribute.

        Values:
            GID_UNSPECIFIED (0):
                GID behavior is unspecified.
            GID_SKIP (1):
                Do not preserve GID during a transfer job.
            GID_NUMBER (2):
                Preserve GID during a transfer job.
        """
        GID_UNSPECIFIED = 0
        GID_SKIP = 1
        GID_NUMBER = 2

    class UID(proto.Enum):
        r"""Options for handling file UID attribute.

        Values:
            UID_UNSPECIFIED (0):
                UID behavior is unspecified.
            UID_SKIP (1):
                Do not preserve UID during a transfer job.
            UID_NUMBER (2):
                Preserve UID during a transfer job.
        """
        UID_UNSPECIFIED = 0
        UID_SKIP = 1
        UID_NUMBER = 2

    class Acl(proto.Enum):
        r"""Options for handling Cloud Storage object ACLs.

        Values:
            ACL_UNSPECIFIED (0):
                ACL behavior is unspecified.
            ACL_DESTINATION_BUCKET_DEFAULT (1):
                Use the destination bucket's default object
                ACLS, if applicable.
            ACL_PRESERVE (2):
                Preserve the object's original ACLs. This requires the
                service account to have ``storage.objects.getIamPolicy``
                permission for the source object. `Uniform bucket-level
                access <https://cloud.google.com/storage/docs/uniform-bucket-level-access>`__
                must not be enabled on either the source or destination
                buckets.
        """
        ACL_UNSPECIFIED = 0
        ACL_DESTINATION_BUCKET_DEFAULT = 1
        ACL_PRESERVE = 2

    class StorageClass(proto.Enum):
        r"""Options for handling Google Cloud Storage object storage
        class.

        Values:
            STORAGE_CLASS_UNSPECIFIED (0):
                Storage class behavior is unspecified.
            STORAGE_CLASS_DESTINATION_BUCKET_DEFAULT (1):
                Use the destination bucket's default storage
                class.
            STORAGE_CLASS_PRESERVE (2):
                Preserve the object's original storage class. This is only
                supported for transfers from Google Cloud Storage buckets.
                REGIONAL and MULTI_REGIONAL storage classes will be mapped
                to STANDARD to ensure they can be written to the destination
                bucket.
            STORAGE_CLASS_STANDARD (3):
                Set the storage class to STANDARD.
            STORAGE_CLASS_NEARLINE (4):
                Set the storage class to NEARLINE.
            STORAGE_CLASS_COLDLINE (5):
                Set the storage class to COLDLINE.
            STORAGE_CLASS_ARCHIVE (6):
                Set the storage class to ARCHIVE.
        """
        STORAGE_CLASS_UNSPECIFIED = 0
        STORAGE_CLASS_DESTINATION_BUCKET_DEFAULT = 1
        STORAGE_CLASS_PRESERVE = 2
        STORAGE_CLASS_STANDARD = 3
        STORAGE_CLASS_NEARLINE = 4
        STORAGE_CLASS_COLDLINE = 5
        STORAGE_CLASS_ARCHIVE = 6

    class TemporaryHold(proto.Enum):
        r"""Options for handling temporary holds for Google Cloud Storage
        objects.

        Values:
            TEMPORARY_HOLD_UNSPECIFIED (0):
                Temporary hold behavior is unspecified.
            TEMPORARY_HOLD_SKIP (1):
                Do not set a temporary hold on the
                destination object.
            TEMPORARY_HOLD_PRESERVE (2):
                Preserve the object's original temporary hold
                status.
        """
        TEMPORARY_HOLD_UNSPECIFIED = 0
        TEMPORARY_HOLD_SKIP = 1
        TEMPORARY_HOLD_PRESERVE = 2

    class KmsKey(proto.Enum):
        r"""Options for handling the KmsKey setting for Google Cloud
        Storage objects.

        Values:
            KMS_KEY_UNSPECIFIED (0):
                KmsKey behavior is unspecified.
            KMS_KEY_DESTINATION_BUCKET_DEFAULT (1):
                Use the destination bucket's default
                encryption settings.
            KMS_KEY_PRESERVE (2):
                Preserve the object's original Cloud KMS
                customer-managed encryption key (CMEK) if
                present. Objects that do not use a Cloud KMS
                encryption key will be encrypted using the
                destination bucket's encryption settings.
        """
        KMS_KEY_UNSPECIFIED = 0
        KMS_KEY_DESTINATION_BUCKET_DEFAULT = 1
        KMS_KEY_PRESERVE = 2

    class TimeCreated(proto.Enum):
        r"""Options for handling ``timeCreated`` metadata for Google Cloud
        Storage objects.

        Values:
            TIME_CREATED_UNSPECIFIED (0):
                TimeCreated behavior is unspecified.
            TIME_CREATED_SKIP (1):
                Do not preserve the ``timeCreated`` metadata from the source
                object.
            TIME_CREATED_PRESERVE_AS_CUSTOM_TIME (2):
                Preserves the source object's ``timeCreated`` metadata in
                the ``customTime`` field in the destination object. Note
                that any value stored in the source object's ``customTime``
                field will not be propagated to the destination object.
        """
        TIME_CREATED_UNSPECIFIED = 0
        TIME_CREATED_SKIP = 1
        TIME_CREATED_PRESERVE_AS_CUSTOM_TIME = 2

    symlink: Symlink = proto.Field(
        proto.ENUM,
        number=1,
        enum=Symlink,
    )
    mode: Mode = proto.Field(
        proto.ENUM,
        number=2,
        enum=Mode,
    )
    gid: GID = proto.Field(
        proto.ENUM,
        number=3,
        enum=GID,
    )
    uid: UID = proto.Field(
        proto.ENUM,
        number=4,
        enum=UID,
    )
    acl: Acl = proto.Field(
        proto.ENUM,
        number=5,
        enum=Acl,
    )
    storage_class: StorageClass = proto.Field(
        proto.ENUM,
        number=6,
        enum=StorageClass,
    )
    temporary_hold: TemporaryHold = proto.Field(
        proto.ENUM,
        number=7,
        enum=TemporaryHold,
    )
    kms_key: KmsKey = proto.Field(
        proto.ENUM,
        number=8,
        enum=KmsKey,
    )
    time_created: TimeCreated = proto.Field(
        proto.ENUM,
        number=9,
        enum=TimeCreated,
    )


class TransferManifest(proto.Message):
    r"""Specifies where the manifest is located.

    Attributes:
        location (str):
            Specifies the path to the manifest in Cloud Storage. The
            Google-managed service account for the transfer must have
            ``storage.objects.get`` permission for this object. An
            example path is ``gs://bucket_name/path/manifest.csv``.
    """

    location: str = proto.Field(
        proto.STRING,
        number=1,
    )


class Schedule(proto.Message):
    r"""Transfers can be scheduled to recur or to run just once.

    Attributes:
        schedule_start_date (google.type.date_pb2.Date):
            Required. The start date of a transfer. Date boundaries are
            determined relative to UTC time. If ``schedule_start_date``
            and
            [start_time_of_day][google.storagetransfer.v1.Schedule.start_time_of_day]
            are in the past relative to the job's creation time, the
            transfer starts the day after you schedule the transfer
            request.

            **Note:** When starting jobs at or near midnight UTC it is
            possible that a job starts later than expected. For example,
            if you send an outbound request on June 1 one millisecond
            prior to midnight UTC and the Storage Transfer Service
            server receives the request on June 2, then it creates a
            TransferJob with ``schedule_start_date`` set to June 2 and a
            ``start_time_of_day`` set to midnight UTC. The first
            scheduled
            [TransferOperation][google.storagetransfer.v1.TransferOperation]
            takes place on June 3 at midnight UTC.
        schedule_end_date (google.type.date_pb2.Date):
            The last day a transfer runs. Date boundaries are determined
            relative to UTC time. A job runs once per 24 hours within
            the following guidelines:

            -  If ``schedule_end_date`` and
               [schedule_start_date][google.storagetransfer.v1.Schedule.schedule_start_date]
               are the same and in the future relative to UTC, the
               transfer is executed only one time.
            -  If ``schedule_end_date`` is later than
               ``schedule_start_date`` and ``schedule_end_date`` is in
               the future relative to UTC, the job runs each day at
               [start_time_of_day][google.storagetransfer.v1.Schedule.start_time_of_day]
               through ``schedule_end_date``.
        start_time_of_day (google.type.timeofday_pb2.TimeOfDay):
            The time in UTC that a transfer job is scheduled to run.
            Transfers may start later than this time.

            If ``start_time_of_day`` is not specified:

            -  One-time transfers run immediately.
            -  Recurring transfers run immediately, and each day at
               midnight UTC, through
               [schedule_end_date][google.storagetransfer.v1.Schedule.schedule_end_date].

            If ``start_time_of_day`` is specified:

            -  One-time transfers run at the specified time.
            -  Recurring transfers run at the specified time each day,
               through ``schedule_end_date``.
        end_time_of_day (google.type.timeofday_pb2.TimeOfDay):
            The time in UTC that no further transfer operations are
            scheduled. Combined with
            [schedule_end_date][google.storagetransfer.v1.Schedule.schedule_end_date],
            ``end_time_of_day`` specifies the end date and time for
            starting new transfer operations. This field must be greater
            than or equal to the timestamp corresponding to the
            combintation of
            [schedule_start_date][google.storagetransfer.v1.Schedule.schedule_start_date]
            and
            [start_time_of_day][google.storagetransfer.v1.Schedule.start_time_of_day],
            and is subject to the following:

            -  If ``end_time_of_day`` is not set and
               ``schedule_end_date`` is set, then a default value of
               ``23:59:59`` is used for ``end_time_of_day``.

            -  If ``end_time_of_day`` is set and ``schedule_end_date``
               is not set, then
               [INVALID_ARGUMENT][google.rpc.Code.INVALID_ARGUMENT] is
               returned.
        repeat_interval (google.protobuf.duration_pb2.Duration):
            Interval between the start of each scheduled
            TransferOperation. If unspecified, the default
            value is 24 hours. This value may not be less
            than 1 hour.
    """

    schedule_start_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=1,
        message=date_pb2.Date,
    )
    schedule_end_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=2,
        message=date_pb2.Date,
    )
    start_time_of_day: timeofday_pb2.TimeOfDay = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timeofday_pb2.TimeOfDay,
    )
    end_time_of_day: timeofday_pb2.TimeOfDay = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timeofday_pb2.TimeOfDay,
    )
    repeat_interval: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=5,
        message=duration_pb2.Duration,
    )


class EventStream(proto.Message):
    r"""Specifies the Event-driven transfer options. Event-driven
    transfers listen to an event stream to transfer updated files.

    Attributes:
        name (str):
            Required. Specifies a unique name of the resource such as
            AWS SQS ARN in the form
            'arn:aws:sqs:region:account_id:queue_name', or Pub/Sub
            subscription resource name in the form
            'projects/{project}/subscriptions/{sub}'.
        event_stream_start_time (google.protobuf.timestamp_pb2.Timestamp):
            Specifies the date and time that Storage
            Transfer Service starts listening for events
            from this stream. If no start time is specified
            or start time is in the past, Storage Transfer
            Service starts listening immediately.
        event_stream_expiration_time (google.protobuf.timestamp_pb2.Timestamp):
            Specifies the data and time at which Storage
            Transfer Service stops listening for events from
            this stream. After this time, any transfers in
            progress will complete, but no new transfers are
            initiated.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    event_stream_start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    event_stream_expiration_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class TransferJob(proto.Message):
    r"""This resource represents the configuration of a transfer job
    that runs periodically.

    Attributes:
        name (str):
            A unique name (within the transfer project) assigned when
            the job is created. If this field is empty in a
            CreateTransferJobRequest, Storage Transfer Service assigns a
            unique name. Otherwise, the specified name is used as the
            unique name for this job.

            If the specified name is in use by a job, the creation
            request fails with an
            [ALREADY_EXISTS][google.rpc.Code.ALREADY_EXISTS] error.

            This name must start with ``"transferJobs/"`` prefix and end
            with a letter or a number, and should be no more than 128
            characters. For transfers involving PosixFilesystem, this
            name must start with ``transferJobs/OPI`` specifically. For
            all other transfer types, this name must not start with
            ``transferJobs/OPI``.

            Non-PosixFilesystem example:
            ``"transferJobs/^(?!OPI)[A-Za-z0-9-._~]*[A-Za-z0-9]$"``

            PosixFilesystem example:
            ``"transferJobs/OPI^[A-Za-z0-9-._~]*[A-Za-z0-9]$"``

            Applications must not rely on the enforcement of naming
            requirements involving OPI.

            Invalid job names fail with an
            [INVALID_ARGUMENT][google.rpc.Code.INVALID_ARGUMENT] error.
        description (str):
            A description provided by the user for the
            job. Its max length is 1024 bytes when
            Unicode-encoded.
        project_id (str):
            The ID of the Google Cloud project that owns
            the job.
        transfer_spec (google.cloud.storage_transfer_v1.types.TransferSpec):
            Transfer specification.
        notification_config (google.cloud.storage_transfer_v1.types.NotificationConfig):
            Notification configuration. This is not
            supported for transfers involving
            PosixFilesystem.
        logging_config (google.cloud.storage_transfer_v1.types.LoggingConfig):
            Logging configuration.
        schedule (google.cloud.storage_transfer_v1.types.Schedule):
            Specifies schedule for the transfer job.
            This is an optional field. When the field is not
            set, the job never executes a transfer, unless
            you invoke RunTransferJob or update the job to
            have a non-empty schedule.
        event_stream (google.cloud.storage_transfer_v1.types.EventStream):
            Specifies the event stream for the transfer
            job for event-driven transfers. When EventStream
            is specified, the Schedule fields are ignored.
        status (google.cloud.storage_transfer_v1.types.TransferJob.Status):
            Status of the job. This value MUST be specified for
            ``CreateTransferJobRequests``.

            **Note:** The effect of the new job status takes place
            during a subsequent job run. For example, if you change the
            job status from
            [ENABLED][google.storagetransfer.v1.TransferJob.Status.ENABLED]
            to
            [DISABLED][google.storagetransfer.v1.TransferJob.Status.DISABLED],
            and an operation spawned by the transfer is running, the
            status change would not affect the current operation.
        creation_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time that the transfer job
            was created.
        last_modification_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time that the transfer job
            was last modified.
        deletion_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time that the transfer job
            was deleted.
        latest_operation_name (str):
            The name of the most recently started
            TransferOperation of this JobConfig. Present if
            a TransferOperation has been created for this
            JobConfig.
    """

    class Status(proto.Enum):
        r"""The status of the transfer job.

        Values:
            STATUS_UNSPECIFIED (0):
                Zero is an illegal value.
            ENABLED (1):
                New transfers are performed based on the
                schedule.
            DISABLED (2):
                New transfers are not scheduled.
            DELETED (3):
                This is a soft delete state. After a transfer job is set to
                this state, the job and all the transfer executions are
                subject to garbage collection. Transfer jobs become eligible
                for garbage collection 30 days after their status is set to
                ``DELETED``.
        """
        STATUS_UNSPECIFIED = 0
        ENABLED = 1
        DISABLED = 2
        DELETED = 3

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    description: str = proto.Field(
        proto.STRING,
        number=2,
    )
    project_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    transfer_spec: "TransferSpec" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="TransferSpec",
    )
    notification_config: "NotificationConfig" = proto.Field(
        proto.MESSAGE,
        number=11,
        message="NotificationConfig",
    )
    logging_config: "LoggingConfig" = proto.Field(
        proto.MESSAGE,
        number=14,
        message="LoggingConfig",
    )
    schedule: "Schedule" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="Schedule",
    )
    event_stream: "EventStream" = proto.Field(
        proto.MESSAGE,
        number=15,
        message="EventStream",
    )
    status: Status = proto.Field(
        proto.ENUM,
        number=6,
        enum=Status,
    )
    creation_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    last_modification_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=8,
        message=timestamp_pb2.Timestamp,
    )
    deletion_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=9,
        message=timestamp_pb2.Timestamp,
    )
    latest_operation_name: str = proto.Field(
        proto.STRING,
        number=12,
    )


class ErrorLogEntry(proto.Message):
    r"""An entry describing an error that has occurred.

    Attributes:
        url (str):
            Required. A URL that refers to the target (a
            data source, a data sink, or an object) with
            which the error is associated.
        error_details (MutableSequence[str]):
            A list of messages that carry the error
            details.
    """

    url: str = proto.Field(
        proto.STRING,
        number=1,
    )
    error_details: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class ErrorSummary(proto.Message):
    r"""A summary of errors by error code, plus a count and sample
    error log entries.

    Attributes:
        error_code (google.rpc.code_pb2.Code):
            Required.
        error_count (int):
            Required. Count of this type of error.
        error_log_entries (MutableSequence[google.cloud.storage_transfer_v1.types.ErrorLogEntry]):
            Error samples.

            At most 5 error log entries are recorded for a
            given error code for a single transfer
            operation.
    """

    error_code: code_pb2.Code = proto.Field(
        proto.ENUM,
        number=1,
        enum=code_pb2.Code,
    )
    error_count: int = proto.Field(
        proto.INT64,
        number=2,
    )
    error_log_entries: MutableSequence["ErrorLogEntry"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="ErrorLogEntry",
    )


class TransferCounters(proto.Message):
    r"""A collection of counters that report the progress of a
    transfer operation.

    Attributes:
        objects_found_from_source (int):
            Objects found in the data source that are
            scheduled to be transferred, excluding any that
            are filtered based on object conditions or
            skipped due to sync.
        bytes_found_from_source (int):
            Bytes found in the data source that are
            scheduled to be transferred, excluding any that
            are filtered based on object conditions or
            skipped due to sync.
        objects_found_only_from_sink (int):
            Objects found only in the data sink that are
            scheduled to be deleted.
        bytes_found_only_from_sink (int):
            Bytes found only in the data sink that are
            scheduled to be deleted.
        objects_from_source_skipped_by_sync (int):
            Objects in the data source that are not
            transferred because they already exist in the
            data sink.
        bytes_from_source_skipped_by_sync (int):
            Bytes in the data source that are not
            transferred because they already exist in the
            data sink.
        objects_copied_to_sink (int):
            Objects that are copied to the data sink.
        bytes_copied_to_sink (int):
            Bytes that are copied to the data sink.
        objects_deleted_from_source (int):
            Objects that are deleted from the data
            source.
        bytes_deleted_from_source (int):
            Bytes that are deleted from the data source.
        objects_deleted_from_sink (int):
            Objects that are deleted from the data sink.
        bytes_deleted_from_sink (int):
            Bytes that are deleted from the data sink.
        objects_from_source_failed (int):
            Objects in the data source that failed to be
            transferred or that failed to be deleted after
            being transferred.
        bytes_from_source_failed (int):
            Bytes in the data source that failed to be
            transferred or that failed to be deleted after
            being transferred.
        objects_failed_to_delete_from_sink (int):
            Objects that failed to be deleted from the
            data sink.
        bytes_failed_to_delete_from_sink (int):
            Bytes that failed to be deleted from the data
            sink.
        directories_found_from_source (int):
            For transfers involving PosixFilesystem only.

            Number of directories found while listing. For example, if
            the root directory of the transfer is ``base/`` and there
            are two other directories, ``a/`` and ``b/`` under this
            directory, the count after listing ``base/``, ``base/a/``
            and ``base/b/`` is 3.
        directories_failed_to_list_from_source (int):
            For transfers involving PosixFilesystem only.

            Number of listing failures for each directory
            found at the source. Potential failures when
            listing a directory include permission failure
            or block failure. If listing a directory fails,
            no files in the directory are transferred.
        directories_successfully_listed_from_source (int):
            For transfers involving PosixFilesystem only.

            Number of successful listings for each directory
            found at the source.
        intermediate_objects_cleaned_up (int):
            Number of successfully cleaned up
            intermediate objects.
        intermediate_objects_failed_cleaned_up (int):
            Number of intermediate objects failed cleaned
            up.
    """

    objects_found_from_source: int = proto.Field(
        proto.INT64,
        number=1,
    )
    bytes_found_from_source: int = proto.Field(
        proto.INT64,
        number=2,
    )
    objects_found_only_from_sink: int = proto.Field(
        proto.INT64,
        number=3,
    )
    bytes_found_only_from_sink: int = proto.Field(
        proto.INT64,
        number=4,
    )
    objects_from_source_skipped_by_sync: int = proto.Field(
        proto.INT64,
        number=5,
    )
    bytes_from_source_skipped_by_sync: int = proto.Field(
        proto.INT64,
        number=6,
    )
    objects_copied_to_sink: int = proto.Field(
        proto.INT64,
        number=7,
    )
    bytes_copied_to_sink: int = proto.Field(
        proto.INT64,
        number=8,
    )
    objects_deleted_from_source: int = proto.Field(
        proto.INT64,
        number=9,
    )
    bytes_deleted_from_source: int = proto.Field(
        proto.INT64,
        number=10,
    )
    objects_deleted_from_sink: int = proto.Field(
        proto.INT64,
        number=11,
    )
    bytes_deleted_from_sink: int = proto.Field(
        proto.INT64,
        number=12,
    )
    objects_from_source_failed: int = proto.Field(
        proto.INT64,
        number=13,
    )
    bytes_from_source_failed: int = proto.Field(
        proto.INT64,
        number=14,
    )
    objects_failed_to_delete_from_sink: int = proto.Field(
        proto.INT64,
        number=15,
    )
    bytes_failed_to_delete_from_sink: int = proto.Field(
        proto.INT64,
        number=16,
    )
    directories_found_from_source: int = proto.Field(
        proto.INT64,
        number=17,
    )
    directories_failed_to_list_from_source: int = proto.Field(
        proto.INT64,
        number=18,
    )
    directories_successfully_listed_from_source: int = proto.Field(
        proto.INT64,
        number=19,
    )
    intermediate_objects_cleaned_up: int = proto.Field(
        proto.INT64,
        number=22,
    )
    intermediate_objects_failed_cleaned_up: int = proto.Field(
        proto.INT64,
        number=23,
    )


class NotificationConfig(proto.Message):
    r"""Specification to configure notifications published to Pub/Sub.
    Notifications are published to the customer-provided topic using the
    following ``PubsubMessage.attributes``:

    -  ``"eventType"``: one of the
       [EventType][google.storagetransfer.v1.NotificationConfig.EventType]
       values
    -  ``"payloadFormat"``: one of the
       [PayloadFormat][google.storagetransfer.v1.NotificationConfig.PayloadFormat]
       values
    -  ``"projectId"``: the
       [project_id][google.storagetransfer.v1.TransferOperation.project_id]
       of the ``TransferOperation``
    -  ``"transferJobName"``: the
       [transfer_job_name][google.storagetransfer.v1.TransferOperation.transfer_job_name]
       of the ``TransferOperation``
    -  ``"transferOperationName"``: the
       [name][google.storagetransfer.v1.TransferOperation.name] of the
       ``TransferOperation``

    The ``PubsubMessage.data`` contains a
    [TransferOperation][google.storagetransfer.v1.TransferOperation]
    resource formatted according to the specified ``PayloadFormat``.

    Attributes:
        pubsub_topic (str):
            Required. The ``Topic.name`` of the Pub/Sub topic to which
            to publish notifications. Must be of the format:
            ``projects/{project}/topics/{topic}``. Not matching this
            format results in an
            [INVALID_ARGUMENT][google.rpc.Code.INVALID_ARGUMENT] error.
        event_types (MutableSequence[google.cloud.storage_transfer_v1.types.NotificationConfig.EventType]):
            Event types for which a notification is
            desired. If empty, send notifications for all
            event types.
        payload_format (google.cloud.storage_transfer_v1.types.NotificationConfig.PayloadFormat):
            Required. The desired format of the
            notification message payloads.
    """

    class EventType(proto.Enum):
        r"""Enum for specifying event types for which notifications are
        to be published.

        Additional event types may be added in the future. Clients
        should either safely ignore unrecognized event types or
        explicitly specify which event types they are prepared to
        accept.

        Values:
            EVENT_TYPE_UNSPECIFIED (0):
                Illegal value, to avoid allowing a default.
            TRANSFER_OPERATION_SUCCESS (1):
                ``TransferOperation`` completed with status
                [SUCCESS][google.storagetransfer.v1.TransferOperation.Status.SUCCESS].
            TRANSFER_OPERATION_FAILED (2):
                ``TransferOperation`` completed with status
                [FAILED][google.storagetransfer.v1.TransferOperation.Status.FAILED].
            TRANSFER_OPERATION_ABORTED (3):
                ``TransferOperation`` completed with status
                [ABORTED][google.storagetransfer.v1.TransferOperation.Status.ABORTED].
        """
        EVENT_TYPE_UNSPECIFIED = 0
        TRANSFER_OPERATION_SUCCESS = 1
        TRANSFER_OPERATION_FAILED = 2
        TRANSFER_OPERATION_ABORTED = 3

    class PayloadFormat(proto.Enum):
        r"""Enum for specifying the format of a notification message's
        payload.

        Values:
            PAYLOAD_FORMAT_UNSPECIFIED (0):
                Illegal value, to avoid allowing a default.
            NONE (1):
                No payload is included with the notification.
            JSON (2):
                ``TransferOperation`` is `formatted as a JSON
                response <https://developers.google.com/protocol-buffers/docs/proto3#json>`__,
                in application/json.
        """
        PAYLOAD_FORMAT_UNSPECIFIED = 0
        NONE = 1
        JSON = 2

    pubsub_topic: str = proto.Field(
        proto.STRING,
        number=1,
    )
    event_types: MutableSequence[EventType] = proto.RepeatedField(
        proto.ENUM,
        number=2,
        enum=EventType,
    )
    payload_format: PayloadFormat = proto.Field(
        proto.ENUM,
        number=3,
        enum=PayloadFormat,
    )


class LoggingConfig(proto.Message):
    r"""Specifies the logging behavior for transfer operations.

    For cloud-to-cloud transfers, logs are sent to Cloud Logging. See
    `Read transfer
    logs <https://cloud.google.com/storage-transfer/docs/read-transfer-logs>`__
    for details.

    For transfers to or from a POSIX file system, logs are stored in the
    Cloud Storage bucket that is the source or sink of the transfer. See
    [Managing Transfer for on-premises jobs]
    (https://cloud.google.com/storage-transfer/docs/managing-on-prem-jobs#viewing-logs)
    for details.

    Attributes:
        log_actions (MutableSequence[google.cloud.storage_transfer_v1.types.LoggingConfig.LoggableAction]):
            Specifies the actions to be logged. If empty, no logs are
            generated. Not supported for transfers with PosixFilesystem
            data sources; use
            [enable_onprem_gcs_transfer_logs][google.storagetransfer.v1.LoggingConfig.enable_onprem_gcs_transfer_logs]
            instead.
        log_action_states (MutableSequence[google.cloud.storage_transfer_v1.types.LoggingConfig.LoggableActionState]):
            States in which ``log_actions`` are logged. If empty, no
            logs are generated. Not supported for transfers with
            PosixFilesystem data sources; use
            [enable_onprem_gcs_transfer_logs][google.storagetransfer.v1.LoggingConfig.enable_onprem_gcs_transfer_logs]
            instead.
        enable_onprem_gcs_transfer_logs (bool):
            For transfers with a PosixFilesystem source,
            this option enables the Cloud Storage transfer
            logs for this transfer.
    """

    class LoggableAction(proto.Enum):
        r"""Loggable actions.

        Values:
            LOGGABLE_ACTION_UNSPECIFIED (0):
                Default value. This value is unused.
            FIND (1):
                Listing objects in a bucket.
            DELETE (2):
                Deleting objects at the source or the
                destination.
            COPY (3):
                Copying objects to Google Cloud Storage.
        """
        LOGGABLE_ACTION_UNSPECIFIED = 0
        FIND = 1
        DELETE = 2
        COPY = 3

    class LoggableActionState(proto.Enum):
        r"""Loggable action states.

        Values:
            LOGGABLE_ACTION_STATE_UNSPECIFIED (0):
                Default value. This value is unused.
            SUCCEEDED (1):
                ``LoggableAction`` completed successfully. ``SUCCEEDED``
                actions are logged as
                [INFO][google.logging.type.LogSeverity.INFO].
            FAILED (2):
                ``LoggableAction`` terminated in an error state. ``FAILED``
                actions are logged as
                [ERROR][google.logging.type.LogSeverity.ERROR].
        """
        LOGGABLE_ACTION_STATE_UNSPECIFIED = 0
        SUCCEEDED = 1
        FAILED = 2

    log_actions: MutableSequence[LoggableAction] = proto.RepeatedField(
        proto.ENUM,
        number=1,
        enum=LoggableAction,
    )
    log_action_states: MutableSequence[LoggableActionState] = proto.RepeatedField(
        proto.ENUM,
        number=2,
        enum=LoggableActionState,
    )
    enable_onprem_gcs_transfer_logs: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class TransferOperation(proto.Message):
    r"""A description of the execution of a transfer.

    Attributes:
        name (str):
            A globally unique ID assigned by the system.
        project_id (str):
            The ID of the Google Cloud project that owns
            the operation.
        transfer_spec (google.cloud.storage_transfer_v1.types.TransferSpec):
            Transfer specification.
        notification_config (google.cloud.storage_transfer_v1.types.NotificationConfig):
            Notification configuration.
        logging_config (google.cloud.storage_transfer_v1.types.LoggingConfig):
            Cloud Logging configuration.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Start time of this transfer execution.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            End time of this transfer execution.
        status (google.cloud.storage_transfer_v1.types.TransferOperation.Status):
            Status of the transfer operation.
        counters (google.cloud.storage_transfer_v1.types.TransferCounters):
            Information about the progress of the
            transfer operation.
        error_breakdowns (MutableSequence[google.cloud.storage_transfer_v1.types.ErrorSummary]):
            Summarizes errors encountered with sample
            error log entries.
        transfer_job_name (str):
            The name of the transfer job that triggers
            this transfer operation.
    """

    class Status(proto.Enum):
        r"""The status of a TransferOperation.

        Values:
            STATUS_UNSPECIFIED (0):
                Zero is an illegal value.
            IN_PROGRESS (1):
                In progress.
            PAUSED (2):
                Paused.
            SUCCESS (3):
                Completed successfully.
            FAILED (4):
                Terminated due to an unrecoverable failure.
            ABORTED (5):
                Aborted by the user.
            QUEUED (6):
                Temporarily delayed by the system. No user
                action is required.
            SUSPENDING (7):
                The operation is suspending and draining the
                ongoing work to completion.
        """
        STATUS_UNSPECIFIED = 0
        IN_PROGRESS = 1
        PAUSED = 2
        SUCCESS = 3
        FAILED = 4
        ABORTED = 5
        QUEUED = 6
        SUSPENDING = 7

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    project_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    transfer_spec: "TransferSpec" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="TransferSpec",
    )
    notification_config: "NotificationConfig" = proto.Field(
        proto.MESSAGE,
        number=10,
        message="NotificationConfig",
    )
    logging_config: "LoggingConfig" = proto.Field(
        proto.MESSAGE,
        number=12,
        message="LoggingConfig",
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    status: Status = proto.Field(
        proto.ENUM,
        number=6,
        enum=Status,
    )
    counters: "TransferCounters" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="TransferCounters",
    )
    error_breakdowns: MutableSequence["ErrorSummary"] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message="ErrorSummary",
    )
    transfer_job_name: str = proto.Field(
        proto.STRING,
        number=9,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.storage_transfer_v1.types import transfer_types

__protobuf__ = proto.module(
    package="google.storagetransfer.v1",
    manifest={
        "GetGoogleServiceAccountRequest",
        "CreateTransferJobRequest",
        "UpdateTransferJobRequest",
        "GetTransferJobRequest",
        "DeleteTransferJobRequest",
        "ListTransferJobsRequest",
        "ListTransferJobsResponse",
        "PauseTransferOperationRequest",
        "ResumeTransferOperationRequest",
        "RunTransferJobRequest",
        "CreateAgentPoolRequest",
        "UpdateAgentPoolRequest",
        "GetAgentPoolRequest",
        "DeleteAgentPoolRequest",
        "ListAgentPoolsRequest",
        "ListAgentPoolsResponse",
    },
)


class GetGoogleServiceAccountRequest(proto.Message):
    r"""Request passed to GetGoogleServiceAccount.

    Attributes:
        project_id (str):
            Required. The ID of the Google Cloud project
            that the Google service account is associated
            with.
    """

    project_id: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateTransferJobRequest(proto.Message):
    r"""Request passed to CreateTransferJob.

    Attributes:
        transfer_job (google.cloud.storage_transfer_v1.types.TransferJob):
            Required. The job to create.
    """

    transfer_job: transfer_types.TransferJob = proto.Field(
        proto.MESSAGE,
        number=1,
        message=transfer_types.TransferJob,
    )


class UpdateTransferJobRequest(proto.Message):
    r"""Request passed to UpdateTransferJob.

    Attributes:
        job_name (str):
            Required. The name of job to update.
        project_id (str):
            Required. The ID of the Google Cloud project
            that owns the job.
        transfer_job (google.cloud.storage_transfer_v1.types.TransferJob):
            Required. The job to update. ``transferJob`` is expected to
            specify one or more of five fields:
            [description][google.storagetransfer.v1.TransferJob.description],
            [transfer_spec][google.storagetransfer.v1.TransferJob.transfer_spec],
            [notification_config][google.storagetransfer.v1.TransferJob.notification_config],
            [logging_config][google.storagetransfer.v1.TransferJob.logging_config],
            and [status][google.storagetransfer.v1.TransferJob.status].
            An ``UpdateTransferJobRequest`` that specifies other fields
            are rejected with the error
            [INVALID_ARGUMENT][google.rpc.Code.INVALID_ARGUMENT].
            Updating a job status to
            [DELETED][google.storagetransfer.v1.TransferJob.Status.DELETED]
            requires ``storagetransfer.jobs.delete`` permission.
        update_transfer_job_field_mask (google.protobuf.field_mask_pb2.FieldMask):
            The field mask of the fields in ``transferJob`` that are to
            be updated in this request. Fields in ``transferJob`` that
            can be updated are:
            [description][google.storagetransfer.v1.TransferJob.description],
            [transfer_spec][google.storagetransfer.v1.TransferJob.transfer_spec],
            [notification_config][google.storagetransfer.v1.TransferJob.notification_config],
            [logging_config][google.storagetransfer.v1.TransferJob.logging_config],
            and [status][google.storagetransfer.v1.TransferJob.status].
            To update the ``transfer_spec`` of the job, a complete
            transfer specification must be provided. An incomplete
            specification missing any required fields is rejected with
            the error
            [INVALID_ARGUMENT][google.rpc.Code.INVALID_ARGUMENT].
    """

    job_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    project_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    transfer_job: transfer_types.TransferJob = proto.Field(
        proto.MESSAGE,
        number=3,
        message=transfer_types.TransferJob,
    )
    update_transfer_job_field_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )


class GetTransferJobRequest(proto.Message):
    r"""Request passed to GetTransferJob.

    Attributes:
        job_name (str):
            Required. The job to get.
        project_id (str):
            Required. The ID of the Google Cloud project
            that owns the job.
    """

    job_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    project_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DeleteTransferJobRequest(proto.Message):
    r"""Request passed to DeleteTransferJob.

    Attributes:
        job_name (str):
            Required. The job to delete.
        project_id (str):
            Required. The ID of the Google Cloud project
            that owns the job.
    """

    job_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    project_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ListTransferJobsRequest(proto.Message):
    r"""``projectId``, ``jobNames``, and ``jobStatuses`` are query
    parameters that can be specified when listing transfer jobs.

    Attributes:
        filter (str):
            Required. A list of query parameters specified as JSON text
            in the form of:
            ``{"projectId":"my_project_id", "jobNames":["jobid1","jobid2",...], "jobStatuses":["status1","status2",...]}``

            Since ``jobNames`` and ``jobStatuses`` support multiple
            values, their values must be specified with array notation.
            ``projectId`` is required. ``jobNames`` and ``jobStatuses``
            are optional. The valid values for ``jobStatuses`` are
            case-insensitive:
            [ENABLED][google.storagetransfer.v1.TransferJob.Status.ENABLED],
            [DISABLED][google.storagetransfer.v1.TransferJob.Status.DISABLED],
            and
            [DELETED][google.storagetransfer.v1.TransferJob.Status.DELETED].
        page_size (int):
            The list page size. The max allowed value is
            256.
        page_token (str):
            The list page token.
    """

    filter: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=4,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListTransferJobsResponse(proto.Message):
    r"""Response from ListTransferJobs.

    Attributes:
        transfer_jobs (MutableSequence[google.cloud.storage_transfer_v1.types.TransferJob]):
            A list of transfer jobs.
        next_page_token (str):
            The list next page token.
    """

    @property
    def raw_page(self):
        return self

    transfer_jobs: MutableSequence[transfer_types.TransferJob] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=transfer_types.TransferJob,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class PauseTransferOperationRequest(proto.Message):
    r"""Request passed to PauseTransferOperation.

    Attributes:
        name (str):
            Required. The name of the transfer operation.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ResumeTransferOperationRequest(proto.Message):
    r"""Request passed to ResumeTransferOperation.

    Attributes:
        name (str):
            Required. The name of the transfer operation.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class RunTransferJobRequest(proto.Message):
    r"""Request passed to RunTransferJob.

    Attributes:
        job_name (str):
            Required. The name of the transfer job.
        project_id (str):
            Required. The ID of the Google Cloud project
            that owns the transfer job.
    """

    job_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    project_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CreateAgentPoolRequest(proto.Message):
    r"""Specifies the request passed to CreateAgentPool.

    Attributes:
        project_id (str):
            Required. The ID of the Google Cloud project
            that owns the agent pool.
        agent_pool (google.cloud.storage_transfer_v1.types.AgentPool):
            Required. The agent pool to create.
        agent_pool_id (str):
            Required. The ID of the agent pool to create.

            The ``agent_pool_id`` must meet the following requirements:

            -  Length of 128 characters or less.
            -  Not start with the string ``goog``.
            -  Start with a lowercase ASCII character, followed by:

               -  Zero or more: lowercase Latin alphabet characters,
                  numerals, hyphens (``-``), periods (``.``),
                  underscores (``_``), or tildes (``~``).
               -  One or more numerals or lowercase ASCII characters.

            As expressed by the regular expression:
            ``^(?!goog)[a-z]([a-z0-9-._~]*[a-z0-9])?$``.
    """

    project_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    agent_pool: transfer_types.AgentPool = proto.Field(
        proto.MESSAGE,
        number=2,
        message=transfer_types.AgentPool,
    )
    agent_pool_id: str = proto.Field(
        proto.STRING,
        number=3,
    )


class UpdateAgentPoolRequest(proto.Message):
    r"""Specifies the request passed to UpdateAgentPool.

    Attributes:
        agent_pool (google.cloud.storage_transfer_v1.types.AgentPool):
            Required. The agent pool to update. ``agent_pool`` is
            expected to specify following fields:

            -  [name][google.storagetransfer.v1.AgentPool.name]

            -  [display_name][google.storagetransfer.v1.AgentPool.display_name]

            -  [bandwidth_limit][google.storagetransfer.v1.AgentPool.bandwidth_limit]
               An ``UpdateAgentPoolRequest`` with any other fields is
               rejected with the error
               [INVALID_ARGUMENT][google.rpc.Code.INVALID_ARGUMENT].
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            The [field mask]
            (https://developers.google.com/protocol-buffers/docs/reference/google.protobuf)
            of the fields in ``agentPool`` to update in this request.
            The following ``agentPool`` fields can be updated:

            -  [display_name][google.storagetransfer.v1.AgentPool.display_name]

            -  [bandwidth_limit][google.storagetransfer.v1.AgentPool.bandwidth_limit]
    """

    agent_pool: transfer_types.AgentPool = proto.Field(
        proto.MESSAGE,
        number=1,
        message=transfer_types.AgentPool,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class GetAgentPoolRequest(proto.Message):
    r"""Specifies the request passed to GetAgentPool.

    Attributes:
        name (str):
            Required. The name of the agent pool to get.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class DeleteAgentPoolRequest(proto.Message):
    r"""Specifies the request passed to DeleteAgentPool.

    Attributes:
        name (str):
            Required. The name of the agent pool to
            delete.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListAgentPoolsRequest(proto.Message):
    r"""The request passed to ListAgentPools.

    Attributes:
        project_id (str):
            Required. The ID of the Google Cloud project
            that owns the job.
        filter (str):
            An optional list of query parameters specified as JSON text
            in the form of:

            ``{"agentPoolNames":["agentpool1","agentpool2",...]}``

            Since ``agentPoolNames`` support multiple values, its values
            must be specified with array notation. When the filter is
            either empty or not provided, the list returns all agent
            pools for the project.
        page_size (int):
            The list page size. The max allowed value is ``256``.
        page_token (str):
            The list page token.
    """

    project_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListAgentPoolsResponse(proto.Message):
    r"""Response from ListAgentPools.

    Attributes:
        agent_pools (MutableSequence[google.cloud.storage_transfer_v1.types.AgentPool]):
            A list of agent pools.
        next_page_token (str):
            The list next page token.
    """

    @property
    def raw_page(self):
        return self

    agent_pools: MutableSequence[transfer_types.AgentPool] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=transfer_types.AgentPool,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.storage_transfer_v1 import gapic_version as package_version

__version__ = package_version.__version__


from .services.storage_transfer_service import (
    StorageTransferServiceAsyncClient,
    StorageTransferServiceClient,
)
from .types.transfer import (
    CreateAgentPoolRequest,
    CreateTransferJobRequest,
    DeleteAgentPoolRequest,
    DeleteTransferJobRequest,
    GetAgentPoolRequest,
    GetGoogleServiceAccountRequest,
    GetTransferJobRequest,
    ListAgentPoolsRequest,
    ListAgentPoolsResponse,
    ListTransferJobsRequest,
    ListTransferJobsResponse,
    PauseTransferOperationRequest,
    ResumeTransferOperationRequest,
    RunTransferJobRequest,
    UpdateAgentPoolRequest,
    UpdateTransferJobRequest,
)
from .types.transfer_types import (
    AgentPool,
    AwsAccessKey,
    AwsS3CompatibleData,
    AwsS3Data,
    AzureBlobStorageData,
    AzureCredentials,
    ErrorLogEntry,
    ErrorSummary,
    EventStream,
    GcsData,
    GoogleServiceAccount,
    HttpData,
    LoggingConfig,
    MetadataOptions,
    NotificationConfig,
    ObjectConditions,
    PosixFilesystem,
    S3CompatibleMetadata,
    Schedule,
    TransferCounters,
    TransferJob,
    TransferManifest,
    TransferOperation,
    TransferOptions,
    TransferSpec,
)

__all__ = (
    "StorageTransferServiceAsyncClient",
    "AgentPool",
    "AwsAccessKey",
    "AwsS3CompatibleData",
    "AwsS3Data",
    "AzureBlobStorageData",
    "AzureCredentials",
    "CreateAgentPoolRequest",
    "CreateTransferJobRequest",
    "DeleteAgentPoolRequest",
    "DeleteTransferJobRequest",
    "ErrorLogEntry",
    "ErrorSummary",
    "EventStream",
    "GcsData",
    "GetAgentPoolRequest",
    "GetGoogleServiceAccountRequest",
    "GetTransferJobRequest",
    "GoogleServiceAccount",
    "HttpData",
    "ListAgentPoolsRequest",
    "ListAgentPoolsResponse",
    "ListTransferJobsRequest",
    "ListTransferJobsResponse",
    "LoggingConfig",
    "MetadataOptions",
    "NotificationConfig",
    "ObjectConditions",
    "PauseTransferOperationRequest",
    "PosixFilesystem",
    "ResumeTransferOperationRequest",
    "RunTransferJobRequest",
    "S3CompatibleMetadata",
    "Schedule",
    "StorageTransferServiceClient",
    "TransferCounters",
    "TransferJob",
    "TransferManifest",
    "TransferOperation",
    "TransferOptions",
    "TransferSpec",
    "UpdateAgentPoolRequest",
    "UpdateTransferJobRequest",
)



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import (
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.storage_transfer_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object]  # type: ignore

from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore

from google.cloud.storage_transfer_v1.services.storage_transfer_service import pagers
from google.cloud.storage_transfer_v1.types import transfer, transfer_types

from .client import StorageTransferServiceClient
from .transports.base import DEFAULT_CLIENT_INFO, StorageTransferServiceTransport
from .transports.grpc_asyncio import StorageTransferServiceGrpcAsyncIOTransport


class StorageTransferServiceAsyncClient:
    """Storage Transfer Service and its protos.
    Transfers data between between Google Cloud Storage buckets or
    from a data source external to Google to a Cloud Storage bucket.
    """

    _client: StorageTransferServiceClient

    DEFAULT_ENDPOINT = StorageTransferServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = StorageTransferServiceClient.DEFAULT_MTLS_ENDPOINT

    agent_pools_path = staticmethod(StorageTransferServiceClient.agent_pools_path)
    parse_agent_pools_path = staticmethod(
        StorageTransferServiceClient.parse_agent_pools_path
    )
    common_billing_account_path = staticmethod(
        StorageTransferServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        StorageTransferServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(StorageTransferServiceClient.common_folder_path)
    parse_common_folder_path = staticmethod(
        StorageTransferServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        StorageTransferServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        StorageTransferServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(StorageTransferServiceClient.common_project_path)
    parse_common_project_path = staticmethod(
        StorageTransferServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        StorageTransferServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        StorageTransferServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            StorageTransferServiceAsyncClient: The constructed client.
        """
        return StorageTransferServiceClient.from_service_account_info.__func__(StorageTransferServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            StorageTransferServiceAsyncClient: The constructed client.
        """
        return StorageTransferServiceClient.from_service_account_file.__func__(StorageTransferServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return StorageTransferServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> StorageTransferServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            StorageTransferServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(StorageTransferServiceClient).get_transport_class,
        type(StorageTransferServiceClient),
    )

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[str, StorageTransferServiceTransport] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the storage transfer service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.StorageTransferServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = StorageTransferServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def get_google_service_account(
        self,
        request: Optional[Union[transfer.GetGoogleServiceAccountRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> transfer_types.GoogleServiceAccount:
        r"""Returns the Google service account that is used by
        Storage Transfer Service to access buckets in the
        project where transfers run or in other projects. Each
        Google service account is associated with one Google
        Cloud project. Users
        should add this service account to the Google Cloud
        Storage bucket ACLs to grant access to Storage Transfer
        Service. This service account is created and owned by
        Storage Transfer Service and can only be used by Storage
        Transfer Service.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_get_google_service_account():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.GetGoogleServiceAccountRequest(
                    project_id="project_id_value",
                )

                # Make the request
                response = await client.get_google_service_account(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.GetGoogleServiceAccountRequest, dict]]):
                The request object. Request passed to
                GetGoogleServiceAccount.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.types.GoogleServiceAccount:
                Google service account
        """
        # Create or coerce a protobuf request object.
        request = transfer.GetGoogleServiceAccountRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_google_service_account,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("project_id", request.project_id),)
            ),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def create_transfer_job(
        self,
        request: Optional[Union[transfer.CreateTransferJobRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> transfer_types.TransferJob:
        r"""Creates a transfer job that runs periodically.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_create_transfer_job():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.CreateTransferJobRequest(
                )

                # Make the request
                response = await client.create_transfer_job(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.CreateTransferJobRequest, dict]]):
                The request object. Request passed to CreateTransferJob.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.types.TransferJob:
                This resource represents the
                configuration of a transfer job that
                runs periodically.

        """
        # Create or coerce a protobuf request object.
        request = transfer.CreateTransferJobRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_transfer_job,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def update_transfer_job(
        self,
        request: Optional[Union[transfer.UpdateTransferJobRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> transfer_types.TransferJob:
        r"""Updates a transfer job. Updating a job's transfer spec does not
        affect transfer operations that are running already.

        **Note:** The job's
        [status][google.storagetransfer.v1.TransferJob.status] field can
        be modified using this RPC (for example, to set a job's status
        to
        [DELETED][google.storagetransfer.v1.TransferJob.Status.DELETED],
        [DISABLED][google.storagetransfer.v1.TransferJob.Status.DISABLED],
        or
        [ENABLED][google.storagetransfer.v1.TransferJob.Status.ENABLED]).

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_update_transfer_job():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.UpdateTransferJobRequest(
                    job_name="job_name_value",
                    project_id="project_id_value",
                )

                # Make the request
                response = await client.update_transfer_job(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.UpdateTransferJobRequest, dict]]):
                The request object. Request passed to UpdateTransferJob.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.types.TransferJob:
                This resource represents the
                configuration of a transfer job that
                runs periodically.

        """
        # Create or coerce a protobuf request object.
        request = transfer.UpdateTransferJobRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_transfer_job,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("job_name", request.job_name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_transfer_job(
        self,
        request: Optional[Union[transfer.GetTransferJobRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> transfer_types.TransferJob:
        r"""Gets a transfer job.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_get_transfer_job():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.GetTransferJobRequest(
                    job_name="job_name_value",
                    project_id="project_id_value",
                )

                # Make the request
                response = await client.get_transfer_job(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.GetTransferJobRequest, dict]]):
                The request object. Request passed to GetTransferJob.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.types.TransferJob:
                This resource represents the
                configuration of a transfer job that
                runs periodically.

        """
        # Create or coerce a protobuf request object.
        request = transfer.GetTransferJobRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_transfer_job,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("job_name", request.job_name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_transfer_jobs(
        self,
        request: Optional[Union[transfer.ListTransferJobsRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListTransferJobsAsyncPager:
        r"""Lists transfer jobs.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_list_transfer_jobs():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.ListTransferJobsRequest(
                    filter="filter_value",
                )

                # Make the request
                page_result = client.list_transfer_jobs(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.ListTransferJobsRequest, dict]]):
                The request object. ``projectId``, ``jobNames``, and ``jobStatuses`` are
                query parameters that can be specified when listing
                transfer jobs.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.services.storage_transfer_service.pagers.ListTransferJobsAsyncPager:
                Response from ListTransferJobs.

                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        request = transfer.ListTransferJobsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_transfer_jobs,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListTransferJobsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def pause_transfer_operation(
        self,
        request: Optional[Union[transfer.PauseTransferOperationRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Pauses a transfer operation.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_pause_transfer_operation():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.PauseTransferOperationRequest(
                    name="name_value",
                )

                # Make the request
                await client.pause_transfer_operation(request=request)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.PauseTransferOperationRequest, dict]]):
                The request object. Request passed to
                PauseTransferOperation.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        request = transfer.PauseTransferOperationRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.pause_transfer_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def resume_transfer_operation(
        self,
        request: Optional[Union[transfer.ResumeTransferOperationRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Resumes a transfer operation that is paused.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_resume_transfer_operation():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.ResumeTransferOperationRequest(
                    name="name_value",
                )

                # Make the request
                await client.resume_transfer_operation(request=request)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.ResumeTransferOperationRequest, dict]]):
                The request object. Request passed to
                ResumeTransferOperation.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        request = transfer.ResumeTransferOperationRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.resume_transfer_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def run_transfer_job(
        self,
        request: Optional[Union[transfer.RunTransferJobRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Starts a new operation for the specified transfer job. A
        ``TransferJob`` has a maximum of one active
        ``TransferOperation``. If this method is called while a
        ``TransferOperation`` is active, an error is returned.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_run_transfer_job():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.RunTransferJobRequest(
                    job_name="job_name_value",
                    project_id="project_id_value",
                )

                # Make the request
                operation = client.run_transfer_job(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.RunTransferJobRequest, dict]]):
                The request object. Request passed to RunTransferJob.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

        """
        # Create or coerce a protobuf request object.
        request = transfer.RunTransferJobRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.run_transfer_job,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("job_name", request.job_name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=transfer_types.TransferOperation,
        )

        # Done; return the response.
        return response

    async def delete_transfer_job(
        self,
        request: Optional[Union[transfer.DeleteTransferJobRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes a transfer job. Deleting a transfer job sets its status
        to
        [DELETED][google.storagetransfer.v1.TransferJob.Status.DELETED].

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_delete_transfer_job():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.DeleteTransferJobRequest(
                    job_name="job_name_value",
                    project_id="project_id_value",
                )

                # Make the request
                await client.delete_transfer_job(request=request)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.DeleteTransferJobRequest, dict]]):
                The request object. Request passed to DeleteTransferJob.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        request = transfer.DeleteTransferJobRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_transfer_job,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("job_name", request.job_name),)),
        )

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def create_agent_pool(
        self,
        request: Optional[Union[transfer.CreateAgentPoolRequest, dict]] = None,
        *,
        project_id: Optional[str] = None,
        agent_pool: Optional[transfer_types.AgentPool] = None,
        agent_pool_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> transfer_types.AgentPool:
        r"""Creates an agent pool resource.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_create_agent_pool():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                agent_pool = storage_transfer_v1.AgentPool()
                agent_pool.name = "name_value"

                request = storage_transfer_v1.CreateAgentPoolRequest(
                    project_id="project_id_value",
                    agent_pool=agent_pool,
                    agent_pool_id="agent_pool_id_value",
                )

                # Make the request
                response = await client.create_agent_pool(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.CreateAgentPoolRequest, dict]]):
                The request object. Specifies the request passed to
                CreateAgentPool.
            project_id (:class:`str`):
                Required. The ID of the Google Cloud
                project that owns the agent pool.

                This corresponds to the ``project_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            agent_pool (:class:`google.cloud.storage_transfer_v1.types.AgentPool`):
                Required. The agent pool to create.
                This corresponds to the ``agent_pool`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            agent_pool_id (:class:`str`):
                Required. The ID of the agent pool to create.

                The ``agent_pool_id`` must meet the following
                requirements:

                -  Length of 128 characters or less.
                -  Not start with the string ``goog``.
                -  Start with a lowercase ASCII character, followed by:

                   -  Zero or more: lowercase Latin alphabet characters,
                      numerals, hyphens (``-``), periods (``.``),
                      underscores (``_``), or tildes (``~``).
                   -  One or more numerals or lowercase ASCII
                      characters.

                As expressed by the regular expression:
                ``^(?!goog)[a-z]([a-z0-9-._~]*[a-z0-9])?$``.

                This corresponds to the ``agent_pool_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.types.AgentPool:
                Represents an On-Premises Agent pool.
        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([project_id, agent_pool, agent_pool_id])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = transfer.CreateAgentPoolRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if project_id is not None:
            request.project_id = project_id
        if agent_pool is not None:
            request.agent_pool = agent_pool
        if agent_pool_id is not None:
            request.agent_pool_id = agent_pool_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_agent_pool,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("project_id", request.project_id),)
            ),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def update_agent_pool(
        self,
        request: Optional[Union[transfer.UpdateAgentPoolRequest, dict]] = None,
        *,
        agent_pool: Optional[transfer_types.AgentPool] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> transfer_types.AgentPool:
        r"""Updates an existing agent pool resource.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_update_agent_pool():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                agent_pool = storage_transfer_v1.AgentPool()
                agent_pool.name = "name_value"

                request = storage_transfer_v1.UpdateAgentPoolRequest(
                    agent_pool=agent_pool,
                )

                # Make the request
                response = await client.update_agent_pool(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.UpdateAgentPoolRequest, dict]]):
                The request object. Specifies the request passed to
                UpdateAgentPool.
            agent_pool (:class:`google.cloud.storage_transfer_v1.types.AgentPool`):
                Required. The agent pool to update. ``agent_pool`` is
                expected to specify following fields:

                -  [name][google.storagetransfer.v1.AgentPool.name]

                -  [display_name][google.storagetransfer.v1.AgentPool.display_name]

                -  [bandwidth_limit][google.storagetransfer.v1.AgentPool.bandwidth_limit]
                   An ``UpdateAgentPoolRequest`` with any other fields
                   is rejected with the error
                   [INVALID_ARGUMENT][google.rpc.Code.INVALID_ARGUMENT].

                This corresponds to the ``agent_pool`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                The [field mask]
                (https://developers.google.com/protocol-buffers/docs/reference/google.protobuf)
                of the fields in ``agentPool`` to update in this
                request. The following ``agentPool`` fields can be
                updated:

                -  [display_name][google.storagetransfer.v1.AgentPool.display_name]

                -  [bandwidth_limit][google.storagetransfer.v1.AgentPool.bandwidth_limit]

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.types.AgentPool:
                Represents an On-Premises Agent pool.
        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([agent_pool, update_mask])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = transfer.UpdateAgentPoolRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if agent_pool is not None:
            request.agent_pool = agent_pool
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_agent_pool,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("agent_pool.name", request.agent_pool.name),)
            ),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_agent_pool(
        self,
        request: Optional[Union[transfer.GetAgentPoolRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> transfer_types.AgentPool:
        r"""Gets an agent pool.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_get_agent_pool():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.GetAgentPoolRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_agent_pool(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.GetAgentPoolRequest, dict]]):
                The request object. Specifies the request passed to
                GetAgentPool.
            name (:class:`str`):
                Required. The name of the agent pool
                to get.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.types.AgentPool:
                Represents an On-Premises Agent pool.
        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = transfer.GetAgentPoolRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_agent_pool,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_agent_pools(
        self,
        request: Optional[Union[transfer.ListAgentPoolsRequest, dict]] = None,
        *,
        project_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListAgentPoolsAsyncPager:
        r"""Lists agent pools.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_list_agent_pools():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.ListAgentPoolsRequest(
                    project_id="project_id_value",
                )

                # Make the request
                page_result = client.list_agent_pools(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.ListAgentPoolsRequest, dict]]):
                The request object. The request passed to ListAgentPools.
            project_id (:class:`str`):
                Required. The ID of the Google Cloud
                project that owns the job.

                This corresponds to the ``project_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.storage_transfer_v1.services.storage_transfer_service.pagers.ListAgentPoolsAsyncPager:
                Response from ListAgentPools.

                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([project_id])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = transfer.ListAgentPoolsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if project_id is not None:
            request.project_id = project_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_agent_pools,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("project_id", request.project_id),)
            ),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListAgentPoolsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def delete_agent_pool(
        self,
        request: Optional[Union[transfer.DeleteAgentPoolRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes an agent pool.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import storage_transfer_v1

            async def sample_delete_agent_pool():
                # Create a client
                client = storage_transfer_v1.StorageTransferServiceAsyncClient()

                # Initialize request argument(s)
                request = storage_transfer_v1.DeleteAgentPoolRequest(
                    name="name_value",
                )

                # Make the request
                await client.delete_agent_pool(request=request)

        Args:
            request (Optional[Union[google.cloud.storage_transfer_v1.types.DeleteAgentPoolRequest, dict]]):
                The request object. Specifies the request passed to
                DeleteAgentPool.
            name (:class:`str`):
                Required. The name of the agent pool
                to delete.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = transfer.DeleteAgentPoolRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_agent_pool,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def list_operations(
        self,
        request: Optional[operations_pb2.ListOperationsRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operations_pb2.ListOperationsResponse:
        r"""Lists operations that match the specified filter in the request.

        Args:
            request (:class:`~.operations_pb2.ListOperationsRequest`):
                The request object. Request message for
                `ListOperations` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            ~.operations_pb2.ListOperationsResponse:
                Response message for ``ListOperations`` method.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.ListOperationsRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_operations,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_operation(
        self,
        request: Optional[operations_pb2.GetOperationRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operations_pb2.Operation:
        r"""Gets the latest state of a long-running operation.

        Args:
            request (:class:`~.operations_pb2.GetOperationRequest`):
                The request object. Request message for
                `GetOperation` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            ~.operations_pb2.Operation:
                An ``Operation`` object.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.GetOperationRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def cancel_operation(
        self,
        request: Optional[operations_pb2.CancelOperationRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Starts asynchronous cancellation on a long-running operation.

        The server makes a best effort to cancel the operation, but success
        is not guaranteed.  If the server doesn't support this method, it returns
        `google.rpc.Code.UNIMPLEMENTED`.

        Args:
            request (:class:`~.operations_pb2.CancelOperationRequest`):
                The request object. Request message for
                `CancelOperation` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            None
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.CancelOperationRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.cancel_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def __aenter__(self) -> "StorageTransferServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("StorageTransferServiceAsyncClient",)
